;;; w3-e20.el --- Emacs 20 specific functions for emacs-w3
;; Author: K.Handa <handa@etl.go.jp>
;; Created: 1997/01/21 10:00:00
;; Version: 1.0
;; Keywords: faces, help, i18n, mouse, hypermedia

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 1997 by Electrotechnical Laboratory
;;;
;;; This file is part of GNU Emacs.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Multi-Lingual Emacs (Emacs 20) Specific Functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconst w3-mime-list-for-code-conversion
  '("text/plain" "text/html")
  "List of MIME types that require Emacs' code conversion.")

(defconst w3-url-coding-system-list
  '(("\\.th$" . th-tis620)
    ("\\.kr$" . euc-kr)
    ("\\.cn$" . cn-gb-2312)
    ("\\.tw$" . cn-big5)
    ("\\.hk$" . cn-big5)
    ("\\.su$" . koi8)
    ("\\.il$" . iso-8859-8)
    ("\\.fr$" . iso-8859-1)
    ))

(defconst w3-mime-charset-list
  '(("euc-kr" . euc-kr)
    ("iso-2022-jp" . iso-2022-jp)
    ("x-sjis" . shift-jis)
    ("gb" . cn-gb-2312)
    ("big5" . cn-big5)
    ("iso-8859-1" . iso-8859-1)
    ("koi-8" . koi8)
    ))

(defvar w3-coding-system-for-read nil)

(defun w3-convert-code-for-emacs20 (mmtype)
  "Convert current data into the appropriate coding system"
  (if (eq w3-coding-system-for-read t)
      (setq w3-coding-system-for-read
	    (read-coding-system "Coding system: ")))
  (let ((coding w3-coding-system-for-read))
    (if (not coding)
	(if mmtype
	    (let ((l w3-mime-list-for-code-conversion))
	      (while l
		(if (string-match (car l) mmtype)
		    (let (charset-field)
		      (if (string-match "charset=\\(.*\\)" mmtype)
			  (let ((ll w3-mime-charset-list))
			    (setq charset-field (match-string 1 mmtype))
			    (while ll
			      (if (string-match (car (car ll)) charset-field)
				  (setq coding (cdr (car ll)) ll nil)
				(setq ll (cdr ll))))
			    (or coding (setq coding 'automatic-conversion)))
			(setq coding 'automatic-conversion))
		      (setq l nil)))
		(setq l (cdr l))))
	  (setq coding 'automatic-conversion)))
    (if coding
	(let ((enable-multibyte-characters t))
	  (if (and (eq coding 'automatic-conversion)
		   url-current-server)
	      (let ((l w3-url-coding-system-list))
		(while l
		  (if (string-match (car (car l)) url-current-server)
		      (setq coding (cdr (car l)) l nil)
		    (setq l (cdr l))))))
	  (decode-coding-region (point-min) (point-max) coding)
	  (set-buffer-file-coding-system coding)))))

(defun w3-reload-document-emacs20 (&optional coding-system)
  "Reload the current document.
A prefix argument enables user to specify the coding-system interactively."
  (interactive "ZCoding-system: ")
  (let ((w3-coding-system-for-read coding-system))
    (w3-reload-document)))

(defun w3-follow-link-e20 (&optional coding-system)
  "Attempt to follow the hypertext reference under point.
A prefix argument enables user to specify the coding-system interactively."
  (interactive "ZCoding-system: ")
  (let ((w3-coding-system-for-read coding-system))
    (w3-follow-link)))

(define-key w3-mode-map "g"        'w3-reload-document-emacs20)
(define-key w3-mode-map "\r"       'w3-follow-link-e20)

(provide 'w3-e20)
